"""Dialogue to pick a file.
"""
# imports
from kivy.properties import StringProperty
from dialogue import Dialogue
from kivy.lang import Builder
from os.path import join, dirname, exists, isfile

from dlg_text import LabelDialogue
from dlg_error import ErrorDialogue

# exports
__all__ = ('FileCreatorDialogue', )

# Load kv
Builder.load_file(join(dirname(__file__), 'dlg_new_file.kv'))

# Classes
class FileCreatorDialogue(Dialogue):
    """Dialogue with a file browser to select a file.

    Pass the default text as **text**.

    >>> FileCreatorDialogue(text='Hello world').open()

    """

    text = StringProperty('')

    def ok(self):
        if exists(self.path):
            if not isfile(self.path):
                dlg = ErrorDialogue(text='Please select a file')
                dlg.open()
            else:
                dlg = LabelDialogue(text='Overwrite the file?')
                dlg.bind(on_ok=lambda obj: super(FileCreatorDialogue, self).ok())
                dlg.open()
        else:
            super(FileCreatorDialogue, self).ok()

## EOF ##
